
/*
 *  cat.c
 *  Copyright (C) 1999 Brad Proctor
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Id: cat.c,v 1.4 1999/06/21 15:04:55 brad Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "version.h"

#define  BUF_SIZE    65536

static char help_text[] =
{
   "Usage: cat [-u][file ...]\n\n"
   "  -u         Print without buffering\n"
   "  --help     Print this help message\n"
   "  --version  Print the program version\n\n"
   "Report bugs to fredlie@sprynet.com\n"
};

static char buf[BUF_SIZE];

/*
 * Main
 */

int main(int argc, char **argv)
{
   int i, c;
   FILE *in;
   int flag_nobuf = 0;
   
   if(argc < 2)
   {
      while(1)
      {
         c = getchar();
         if(c == EOF)
            break;
         putchar(c);
      }
      return (0);
   }
   if(!strcmp(argv[1], "--help"))
   {
      puts(help_text);
      return (0);
   }
   else if(!strcmp(argv[1], "--version"))
   {
      puts("cat (butils) "VERSION_STRING);
      return (0);
   }
   else if(!strcmp(argv[1], "-u"))
      flag_nobuf = 1;
   for(i = 1; i < argc; ++i)
   {
      if((in = fopen(argv[i], "r")) == NULL)
      {
         fprintf(stderr, "%s: Could not open '%s' for reading.\n", 
            argv[0], argv[i]);
         return (1);
      }
      if(flag_nobuf)
         while(1)
         {
            c = getc(in);
            if(c == EOF)
               break;
            putchar(c);
            fflush(in);
         }
      else
         while((c = fread(buf, 1, BUF_SIZE, in)) > 0)
            fwrite(buf, 1, c, stdout);
      fclose(in);
   }
   return (0);
}